package org.processmining.plugins.dcnet;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Image;
import java.awt.Stroke;
import java.util.List;

import javax.swing.JPopupMenu;

import org.processmining.models.cnet.CNetBinding;
import org.processmining.models.cnet.CNetNode;
import org.processmining.models.cnet.CausalNet;
import org.processmining.models.dcnet.DCNet;
import org.processmining.models.dcnet.DataAwareCausalNet;

public class DefaultNodeDecorator implements NodeDecorator {

	private final DataAwareCausalNet dataCausalNet;

	public DefaultNodeDecorator(CausalNet causalNet) {
		this(new DCNet(causalNet));
	}

	public DefaultNodeDecorator(DataAwareCausalNet dataCausalNet) {
		super();
		this.dataCausalNet = dataCausalNet;
	}

	public String getBindingLabel(CNetBinding binding) {
		if (getDataCausalNet().getGuardExpression(binding) != null) {
			return getDataCausalNet().getGuardExpression(binding).toPrettyString(1);
		} else {
			return null;
		}
	}

	public Color getBindingFillColor(CNetBinding binding) {
		return Color.BLACK;
	}

	protected DataAwareCausalNet getDataCausalNet() {
		return dataCausalNet;
	}

	public double getBindingSize(CNetBinding binding) {
		return 4.0d;
	}

	public String getNodeLabel(CNetNode node) {
		return null;
	}

	public Color getNodeFillColor(CNetNode node) {
		return Color.WHITE;
	}

	public JPopupMenu getNodeMenu(CNetNode node) {
		return new JPopupMenu();
	}

	public JPopupMenu getBindingMenu(CNetBinding binding) {
		return new JPopupMenu();
	}

	public Stroke getBindingStroke(CNetBinding binding) {
		return null;
	}

	public BorderStyle getBindingBorderStyle(CNetBinding binding) {
		if (getDataCausalNet().getGuardExpression(binding) != null) {
			return BorderStyle.DOUBLE;
		} else {
			return BorderStyle.NONE;
		}
	}

	public Color getBindingStrokeColor(CNetBinding binding) {
		return new Color(255, 255, 255, 0);
	}

	public Color getBindingBorderColor(CNetBinding binding) {
		return Color.BLACK;
	}

	public Stroke getBindingBorderStroke(CNetBinding binding) {
		return new BasicStroke(1.5f);
	}

	public Color getBindingEdgeColor(CNetBinding binding) {
		return Color.BLACK;
	}

	public float getBindingEdgeWidth(CNetBinding binding) {
		return 1.0f;
	}

	public List<ColorSegment> getNodeColorSegments(CNetNode node) {
		return null;
	}

	public List<TrafficLightSegment> getNodeTrafficLights(CNetNode cnetNode) {
		return null;
	}

	public Image getNodeIcon(CNetNode cnetNode) {
		return null;
	}

}